﻿#region License Information
/* HeuristicLab 
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL) 
 * 
 * This file is part of HeuristicLab. 
 * 
 * HeuristicLab is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version. 
 * 
 * HeuristicLab is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>. 
 */
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using HeuristicLab.Core;
using System.Runtime.Serialization;
using HeuristicLab.Common;
using System.Drawing;

namespace HeuristicLab.Services.Authentication {
   [Item("AuthenticationItem", "Base class for all authentication items.")]
  public partial class AuthenticationItem : IItem {

    public virtual string ItemName {
      get { return ItemAttribute.GetName(this.GetType()); }
    }
    public virtual string ItemDescription {
      get { return ItemAttribute.GetDescription(this.GetType()); }
    }
    public Version ItemVersion {
      get { return ItemAttribute.GetVersion(this.GetType()); }
    }
    public virtual Image ItemImage {
      get {
        if (Modified)
          return HeuristicLab.Common.Resources.VS2008ImageLibrary.DatabaseModified;
        else
          return HeuristicLab.Common.Resources.VS2008ImageLibrary.Database;
      }
    }

    protected AuthenticationItem(AuthenticationItem original, Cloner cloner) {
      cloner.RegisterClonedObject(original, this);
      modified = true;
      Name = original.Name;
      Id = original.Id;
    }

    protected AuthenticationItem() {
      modified = true;
    }


    private bool modified;
    public bool Modified {
      get { return modified; }
      private set {
        if (value != modified) {
          modified = value;
          OnModifiedChanged();
          RaisePropertyChanged("Modified");
          OnItemImageChanged();
          RaisePropertyChanged("ItemImage");
        }
      }
    }

    public void Store() {
      if (AuthenticationClient.Instance.Store(this))
        Modified = false;
    }

    protected void RaisePropertyChanged(string propertyName) {
      OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
      if ((propertyName != "Id") && (propertyName != "Modified")) {
        Modified = true;
      }
    }
  
    protected virtual void OnPropertyChanged(PropertyChangedEventArgs e) {
      PropertyChangedEventHandler handler = PropertyChanged;
      if (handler != null) handler(this, e);
    }

    public event EventHandler ModifiedChanged;
    protected virtual void OnModifiedChanged() {
      EventHandler handler = ModifiedChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    [OnDeserialized]
    private void OnDeserialized(StreamingContext context) {
      modified = false;
    }

    public object Clone() {
      return Clone(new Cloner());
    }
    public virtual IDeepCloneable Clone(Cloner cloner) {
      return new AuthenticationItem(this, cloner);
    }

    public override string ToString() {
      return Name;
    }
    public event EventHandler ItemImageChanged;
    protected virtual void OnItemImageChanged() {
      EventHandler handler = ItemImageChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
    public event EventHandler ToStringChanged;
    protected virtual void OnToStringChanged() {
      EventHandler handler = ToStringChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
  }
}

